@@ -10,6 +10,8 @@
 """
 import re
 import os
+import platform
+
 from collections import defaultdict
 from functools import partial
 
@@ -19,10 +21,17 @@ from logbook.handlers import (
     Handler, StringFormatter, StringFormatterHandlerMixin, StderrHandler)
 from logbook._termcolors import colorize
 from logbook.helpers import PY2, string_types, iteritems, u
-
 from logbook.ticketing import TicketingHandler as DatabaseHandler
 from logbook.ticketing import BackendBase
 
+try:
+    import riemann_client.client
+    import riemann_client.transport
+except ImportError:
+    riemann_client = None
+    #from riemann_client.transport import TCPTransport, UDPTransport, BlankTransport
+
+
 if PY2:
     from urllib import urlencode
     from urlparse import parse_qsl
@@ -220,6 +229,33 @@ class TwitterHandler(Handler, StringFormatterHandlerMixin):
         self.tweet(self.format(record))
 
 
+class SlackHandler(Handler, StringFormatterHandlerMixin):
+
+    """A handler that logs to slack.  Requires that you sign up an
+    application on slack and request an api token.  Furthermore the
+    slacker library has to be installed.
+    """
+
+    def __init__(self, api_token, channel, level=NOTSET, format_string=None, filter=None,
+                 bubble=False):
+
+        Handler.__init__(self, level, filter, bubble)
+        StringFormatterHandlerMixin.__init__(self, format_string)
+        self.api_token = api_token
+
+        try:
+            from slacker import Slacker
+        except ImportError:
+            raise RuntimeError('The slacker library is required for '
+                               'the SlackHandler.')
+
+        self.channel = channel
+        self.slack = Slacker(api_token)
+
+    def emit(self, record):
+        self.slack.chat.post_message(channel=self.channel, text=self.format(record))
+
+
 class JinjaFormatter(object):
     """A formatter object that makes it easy to format using a Jinja 2
     template instead of a format string.
@@ -293,6 +329,17 @@ class ColorizingStreamHandlerMixin(object):
 
     .. _`colorama`: https://pypi.python.org/pypi/colorama
     """
+    _use_color = None
+
+    def force_color(self):
+        """Force colorizing the stream (`should_colorize` will return True)
+        """
+        self._use_color = True
+
+    def forbid_color(self):
+        """Forbid colorizing the stream (`should_colorize` will return False)
+        """
+        self._use_color = False
 
     def should_colorize(self, record):
         """Returns `True` if colorizing should be applied to this
@@ -305,6 +352,8 @@ class ColorizingStreamHandlerMixin(object):
                 import colorama
             except ImportError:
                 return False
+        if self._use_color is not None:
+            return self._use_color
         isatty = getattr(self.stream, 'isatty', None)
         return isatty and isatty()
 
@@ -447,3 +496,79 @@ class DedupHandler(Handler):
                 dispatch = dispatch_record
             dispatch(record)
         self.clear()
+
+
+class RiemannHandler(Handler):
+
+    """
+    A handler that sends logs as events to Riemann.
+    """
+
+    def __init__(self,
+                 host,
+                 port,
+                 message_type="tcp",
+                 ttl=60,
+                 flush_threshold=10,
+                 bubble=False,
+                 filter=None,
+                 level=NOTSET):
+        """
+        :param host: riemann host
+        :param port: riemann port
+        :param message_type: selects transport. Currently available 'tcp' and 'udp'
+        :param ttl: defines time to live in riemann
+        :param flush_threshold: count of events after which we send to riemann
+        """
+        if riemann_client is None:
+            raise NotImplementedError("The Riemann handler requires the riemann_client package") # pragma: no cover
+        Handler.__init__(self, level, filter, bubble)
+        self.host = host
+        self.port = port
+        self.ttl = ttl
+        self.queue = []
+        self.flush_threshold = flush_threshold
+        if message_type == "tcp":
+            self.transport = riemann_client.transport.TCPTransport
+        elif message_type == "udp":
+            self.transport = riemann_client.transport.UDPTransport
+        elif message_type == "test":
+            self.transport = riemann_client.transport.BlankTransport
+        else:
+            msg = ("Currently supported message types for RiemannHandler are: {0}. \
+                    {1} is not supported."
+                   .format(",".join(["tcp", "udp", "test"]), message_type))
+            raise RuntimeError(msg)
+
+    def record_to_event(self, record):
+        from time import time
+        tags = ["log", record.level_name]
+        msg = str(record.exc_info[1]) if record.exc_info else record.msg
+        channel_name = str(record.channel) if record.channel else "unknown"
+        if any([record.level_name == keywords
+                for keywords in ["ERROR", "EXCEPTION"]]):
+            state = "error"
+        else:
+            state = "ok"
+        return {"metric_f": 1.0,
+                "tags": tags,
+                "description": msg,
+                "time": int(time()),
+                "ttl": self.ttl,
+                "host": platform.node(),
+                "service": "{0}.{1}".format(channel_name, os.getpid()),
+                "state": state
+                }
+
+    def _flush_events(self):
+        with riemann_client.client.QueuedClient(self.transport(self.host, self.port)) as cl:
+            for event in self.queue:
+                cl.event(**event)
+            cl.flush()
+        self.queue = []
+
+    def emit(self, record):
+        self.queue.append(self.record_to_event(record))
+
+        if len(self.queue) == self.flush_threshold:
+            self._flush_events()
